@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/makefile.h   3.2.3   edited Aug 14 02:19 $
+** $Id: qt/makefile.h   3.3.3   edited Dec 3 2003 $
 **
 ** Definition of MakefileGenerator class.
 **
@@ -39,13 +39,21 @@
 #include "project.h"
 #include <qtextstream.h>
 
+#ifdef Q_OS_WIN32
+#define QT_POPEN _popen
+#else
+#define QT_POPEN popen
+#endif
+
 class MakefileGenerator
 {
     QString spec;
     bool init_opath_already, init_already, moc_aware, no_io;
     QStringList createObjectList(const QString &var);
     QString build_args();
-    QMap<QString, QString> depHeuristics, depKeyMap, fileFixed;
+    QString dependencyKey(const QString &file) const;
+    QMap<QString, bool> depProcessed;
+    QMap<QString, QString> depHeuristics, fileFixed;
     QMap<QString, QString> mocablesToMOC, mocablesFromMOC;
     QMap<QString, QStringList> depends;
 
@@ -79,7 +87,7 @@ protected:
 
     QString findMocSource(const QString &moc_file) const;
     QString findMocDestination(const QString &src_file) const;
-    QStringList &findDependencies(const QString &file);
+    virtual QStringList &findDependencies(const QString &file);
 
     void setNoIO(bool o);
     bool noIO() const;
@@ -95,6 +103,9 @@ protected:
     void initOutPaths();
     virtual void init();
 
+    //for cross-platform dependent directories
+    virtual void usePlatformDir();
+
     //for installs
     virtual QString defaultInstall(const QString &);
 
@@ -108,7 +119,10 @@ protected:
     virtual bool findLibraries();
     virtual QString findDependency(const QString &);
 
-    QString var(const QString &var);
+    void setProcessedDependencies(const QString &file, bool b);
+    bool processedDependencies(const QString &file);
+
+    virtual QString var(const QString &var);
     QString varGlue(const QString &var, const QString &before, const QString &glue, const QString &after);
     QString varList(const QString &var);
     QString val(const QStringList &varList);
@@ -165,7 +179,7 @@ inline QString MakefileGenerator::defaultInstall(const QString &)
 inline bool MakefileGenerator::findLibraries()
 { return TRUE; }
 
-inline QString MakefileGenerator::findDependency(const QString &dep)
+inline QString MakefileGenerator::findDependency(const QString &)
 { return QString(""); }
 
 inline MakefileGenerator::~MakefileGenerator()
