@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qrtlcodec.cpp   3.2.3   edited Oct 27 20:10 $
+** $Id: qt/qrtlcodec.cpp   3.3.3   edited Jan 6 2004 $
 **
 ** Implementation of QTextCodec class
 **
@@ -36,7 +36,7 @@
 **********************************************************************/
 
 #include "qrtlcodec.h"
-#include "private/qcomplextext_p.h"
+#include <private/qtextengine_p.h>
 
 #ifndef QT_NO_CODEC_HEBREW
 
@@ -416,6 +416,61 @@ const char* QHebrewCodec::mimeName() const
     return "ISO-8859-8";
 }
 
+static QString visualOrder(QString logical, QChar::Direction basicDir)
+{
+    logical.replace(QChar('\n'), QChar(0x2028));
+
+    QTextEngine e(logical, 0);
+    e.direction = basicDir;
+    e.itemize();
+    Q_UINT8 l[256];
+    Q_UINT8 *levels = l;
+    int vo[256];
+    int *visualOrder = vo;
+    int nitems = e.items.size();
+    if (nitems > 255) {
+	levels = new Q_UINT8[nitems];
+	visualOrder = new int[nitems];
+    }
+    int i;
+    for (i = 0; i < nitems; ++i) {
+	//qDebug("item %d bidiLevel=%d", i,  e.items[i].analysis.bidiLevel);
+	levels[i] = e.items[i].analysis.bidiLevel;
+    }
+    e.bidiReorder(nitems, levels, visualOrder);
+
+    QString visual;
+    for (i = 0; i < nitems; ++i) {
+	QScriptItem &si = e.items[visualOrder[i]];
+	QString sub = logical.mid(si.position, e.length(visualOrder[i]));
+	if (si.analysis.bidiLevel % 2) {
+	    // reverse sub
+	    QChar *a = (QChar *)sub.unicode();
+	    QChar *b = a + sub.length() - 1;
+	    while (a < b) {
+		QChar tmp = *a;
+		*a = *b;
+		*b = tmp;
+		++a;
+		--b;
+	    }
+	    a = (QChar *)sub.unicode();
+	    b = a + sub.length();
+	    while (a<b) {
+		*a = a->mirroredChar();
+		++a;
+	    }
+	}
+	visual += sub;
+    }
+    // replace Unicode newline back with  \n to compare.
+    visual.replace(QChar(0x2028), QChar('\n'));
+    if (l != levels) {
+	delete [] levels;
+	delete [] visualOrder;
+    }
+    return visual;
+}
 
 /*!
     \reimp
@@ -480,22 +535,7 @@ QString QHebrewCodec::toUnicode(const char* chars, int len ) const
     if( basicDir == QChar::DirON )
 	basicDir = findBasicDirection(r);
 
-    return QComplexText::bidiReorderString(r);
-#if 0
-
-    QString out;
-    int lineStart = 0;
-    while( lineStart < len ) {
-	// find line
-	int lineEnd = r.find('\n', lineStart);
-	if ( lineEnd == -1 ) lineEnd = len;
-	out += reverseLine(r, lineStart, lineEnd, basicDir);
-	if ( lineEnd < len )
-	    out += "\n";
-	lineStart = lineEnd + 1;
-    }
-    return out;
-#endif
+    return visualOrder(r, basicDir);
 }
 
 /*!
@@ -531,7 +571,7 @@ QCString QHebrewCodec::fromUnicode(const QString& uc, int& lenInOut) const
     } else {
 	QString tmp = uc;
 	tmp.truncate(l);
-	QString vis = QComplexText::bidiReorderString(tmp);
+	QString vis = visualOrder(tmp, QChar::DirON);
 
 	for (int i=0; i<l; i++) {
 	    const QChar ch = vis[i];
