@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qasyncimageio.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qasyncimageio.cpp   3.3.3   edited Jul 27 15:31 $
 **
 ** Implementation of asynchronous image/movie loading classes
 **
@@ -992,6 +992,7 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		accum = 0;
 		bitcount = 0;
 		sp = stack;
+                firstcode = oldcode = 0;
 		needfirst = FALSE;
 		out_of_bounds = FALSE;
 	    }
@@ -1169,12 +1170,6 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 	    count++;
 	    if (count==expectcount) {
 		int loop = hold[0]+hold[1]*256;
-
-		// Why if the extension here, if it is supposed to only
-		// play through once?  We assume that the creator meant
-		// 0, which is infinite.
-		if (loop == 1) loop = 0;
-
 		if (consumer) consumer->setLooping(loop);
 		state=SkipBlockSize; // Ignore further blocks
 	    }
