@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qprinter_unix.cpp   3.2.3   edited Oct 1 14:22 $
+** $Id: qt/qprinter_unix.cpp   3.3.3   edited Jul 30 15:56 $
 **
 ** Implementation of QPrinter class for Unix
 **
@@ -54,6 +54,7 @@ static inline int qt_open(const char *pathname, int flags, mode_t mode)
 #include "qapplication.h"
 #include "qprinter_p.h"
 
+#include <unistd.h> // For ::sleep()
 #include <stdlib.h>
 
 
@@ -63,6 +64,7 @@ static inline int qt_open(const char *pathname, int flags, mode_t mode)
 class QPrinterUnixPrivate : public QPrinterPrivate
 {
 public:
+    bool marginsSpecified;
     uint topMargin;
     uint leftMargin;
     uint bottomMargin;
@@ -135,11 +137,8 @@ QPrinter::QPrinter( PrinterMode m )
 	    res = 600;
     }
 
-    d = 0;
-    QPrinterUnixPrivate *tmp = new QPrinterUnixPrivate;
-    margins( &(tmp->topMargin), &(tmp->leftMargin),
-	     &(tmp->bottomMargin), &(tmp->rightMargin) );
-    d = tmp;
+    d = new QPrinterUnixPrivate;
+    D->marginsSpecified = false;
     d->printerOptions = 0;
     setOptionEnabled( PrintToFile, TRUE );
     setOptionEnabled( PrintPageRange, TRUE );
@@ -294,6 +293,16 @@ static void closeAllOpenFds()
 	::close( i );
 }
 
+
+
+static const char * const psToStr[QPrinter::NPageSize+1] =
+{ "A4", "B5", "Letter", "Legal", "Executive",
+  "A0", "A1", "A2", "A3", "A5", "A6", "A7", "A8", "A9", "B0", "B1",
+  "B10", "B2", "B3", "B4", "B6", "B7", "B8", "B9", "C5E", "Comm10E",
+  "DLE", "Folio", "Ledger", "Tabloid", 0
+};
+
+
 /*!
   \internal
   Handles painter commands to the printer.
@@ -373,7 +382,7 @@ bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
 		    }
 		    dup2( fds[0], 0 );
 
-		    closeAllOpenFds();
+ 		    closeAllOpenFds();
 
 		    if ( print_prog ) {
 		        if ( option_string )
@@ -385,30 +394,45 @@ bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
 		    } else {
 			// if no print program has been specified, be smart
 			// about the option string too.
-			const char * lprarg = 0;
-			QString lprhack;
-			const char * lparg = 0;
-			QString lphack;
+			QStringList lprhack;
+			QStringList lphack;
+                        QString media;
 			if ( pr || option_string ) {
-			    lprhack = pr;
-			    if ( option_string )
-				lprhack.prepend( option_string );
-			    else
-				lprhack.prepend( QString::fromLatin1( "-P" ) );
-			    lprarg = lprhack.ascii();
-			    lphack = pr;
-			    if ( option_string )
-				lphack.prepend( option_string );
-			    else
-				lphack.prepend( QString::fromLatin1( "-d" ) );
-			    lparg = lphack.ascii();
+			    if ( option_string ) {
+				lprhack = QStringList::split(QChar(' '), option_string);
+                                lphack = lprhack;
+			    } else {
+				lprhack.append( QString::fromLatin1( "-P" ) );
+				lphack.append( QString::fromLatin1( "-d" ) );
+                            }
+			    lprhack.append(pr);
+			    lphack.append(pr);
 			}
-			(void)execlp( "lp", "lp", lparg, (char *)0 );
-			(void)execlp( "lpr", "lpr", lprarg, (char *)0 );
-			(void)execl( "/bin/lp", "lp", lparg, (char *)0 );
-			(void)execl( "/bin/lpr", "lpr", lprarg, (char *)0 );
-			(void)execl( "/usr/bin/lp", "lp", lparg, (char *)0 );
-			(void)execl( "/usr/bin/lpr", "lpr", lprarg, (char *)0 );
+                        char ** lpargs = new char *[lphack.size()+6];
+                        lpargs[0] = "lp";
+                        uint i;
+                        for (i = 0; i < lphack.size(); ++i)
+                            lpargs[i+1] = (char *)lphack[i].ascii();
+                        if (psToStr[page_size]) {
+                            lpargs[++i] = "-o";
+                            lpargs[++i] = (char *)psToStr[page_size];
+                            lpargs[++i] = "-o";
+                            media = "media=";
+                            media += psToStr[page_size];
+                            lpargs[++i] = (char *)media.ascii();
+                        }
+                        lpargs[++i] = 0;
+                        char **lprargs = new char *[lprhack.size()+1];
+                        lprargs[0] = "lpr";
+                        for (uint x = 0; x < lprhack.size(); ++x)
+                            lprargs[x+1] = (char *)lprhack[x].ascii();
+                        lprargs[lprhack.size() + 1] = 0;
+			(void)execvp( "lp", lpargs );
+			(void)execvp( "lpr", lprargs );
+			(void)execv( "/bin/lp", lpargs);
+			(void)execv( "/bin/lpr", lprargs);
+			(void)execv( "/usr/bin/lp", lpargs);
+			(void)execv( "/usr/bin/lpr", lprargs);
 		    }
 		    // if we couldn't exec anything, close the fd,
 		    // wait for a second so the parent process (the
@@ -483,12 +507,12 @@ static const PaperSize paperSizes[QPrinter::NPageSize] =
     {  MM(88), MM(125) },       // B7
     {  MM(62), MM(88) },        // B8
     {  MM(44), MM(62) },        // B9
-    {  MM(163), MM(229) },      // C5E
-    {  MM(105), MM(241) },      // Comm10E
-    {  MM(110), MM(220) },      // DLE
-    {  MM(210), MM(330) },      // Folio
-    {  MM(432), MM(279) },      // Ledger
-    {  MM(279), MM(432) },      // Tabloid
+    {  MM(162),    MM(229) },   // C5E
+    {  IN(4.125),  IN(9.5) },   // Comm10E
+    {  MM(110),    MM(220) },   // DLE
+    {  IN(8.5),    IN(13) },    // Folio
+    {  IN(17),     IN(11) },    // Ledger
+    {  IN(11),     IN(17) }     // Tabloid
 };
 
 /*!
@@ -513,7 +537,7 @@ int QPrinter::metric( int m ) const
 	if ( res != 72 )
 	    val = (val * res + 36) / 72;
 	if ( !fullPage() ) {
-	    if ( d )
+	    if ( D->marginsSpecified )
 		val -= D->leftMargin + D->rightMargin;
 	    else
 		val -= 2*margins().width();
@@ -524,7 +548,7 @@ int QPrinter::metric( int m ) const
 	if ( res != 72 )
 	    val = (val * res + 36) / 72;
 	if ( !fullPage() ) {
-	    if ( d )
+	    if ( D->marginsSpecified )
 		val -= D->topMargin + D->bottomMargin;
 	    else
 		val -= 2*margins().height();
@@ -583,7 +607,7 @@ int QPrinter::metric( int m ) const
 */
 QSize QPrinter::margins() const
 {
-    if ( d )
+    if ( D->marginsSpecified )
 	return QSize( D->leftMargin, D->topMargin );
 
     if (orient == Portrait)
@@ -610,7 +634,7 @@ QSize QPrinter::margins() const
 */
 void QPrinter::margins( uint *top, uint *left, uint *bottom, uint *right ) const
 {
-    if ( !d ) {
+    if ( !D->marginsSpecified ) {
 	int x = orient == Portrait ? res/2 : res/3;
 	int y = orient == Portrait ? res/3 : res/2;
 	*top = *bottom = y;
@@ -637,6 +661,7 @@ void QPrinter::setMargins( uint top, uint left, uint bottom, uint right )
     D->leftMargin = left;
     D->bottomMargin = bottom;
     D->rightMargin = right;
+    D->marginsSpecified = TRUE;
 }
 
 #endif
