@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qserversocket.cpp   3.2.3   edited Aug 12 16:24 $
+** $Id: qt/qserversocket.cpp   3.3.3   edited Oct 7 2003 $
 **
 ** Implementation of QServerSocket class.
 **
@@ -156,7 +156,8 @@ bool QServerSocket::ok() const
  */
 void QServerSocket::init( const QHostAddress & address, Q_UINT16 port, int backlog )
 {
-    d->s = new QSocketDevice;
+    d->s = new QSocketDevice( QSocketDevice::Stream, address.isIPv4Address()
+			      ? QSocketDevice::IPv4 : QSocketDevice::IPv6, 0 );
 #if !defined(Q_OS_WIN32)
     // Under Unix, we want to be able to use the port, even if a socket on the
     // same address-port is in TIME_WAIT. Under Windows this is possible anyway
