@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qcomlibrary.cpp   3.2.3   edited Jul 15 09:50 $
+** $Id: qt/qcomlibrary.cpp   3.3.3   edited Nov 11 2003 $
 **
 ** Implementation of QComLibrary class
 **
@@ -73,18 +73,16 @@ QComLibrary::~QComLibrary()
 
 bool QComLibrary::unload()
 {
-    if ( libiface ) {
-	libiface->cleanup();
-	if ( !libiface->canUnload() )
-	    return FALSE;
-	libiface->release();
-	libiface = 0;
-    }
     int refs = entry ? entry->release() : 0;
-    if ( refs )
+    entry = 0;
+    if (refs || !libiface)
 	return FALSE;
 
-    entry = 0;
+    libiface->cleanup();
+    if ( !libiface->canUnload() )
+	return FALSE;
+    libiface->release();
+    libiface = 0;
 
     return QLibrary::unload();
 }
