@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qdatetime.cpp   3.1.2   edited Dec 30 17:45 $
+** $Id: qt/qdatetime.cpp   3.3.3   edited Jan 20 2004 $
 **
 ** Implementation of date and time classes
 **
@@ -149,6 +149,11 @@ static QString fmtDateTime( const QString& f, const QTime* dt = 0, const QDate*
 	return QString::null;
     }
 
+    if ( dt && !dt->isValid() )
+	return QString::null;
+    if ( dd && !dd->isValid() )
+	return QString::null;
+
     bool ap = ( f.contains( "AP" ) || f.contains( "ap" ) );
 
     QString buf;
@@ -402,20 +407,37 @@ int QDate::daysInYear() const
 }
 
 /*!
-  Returns the week number (1 to 53), and stores the year in \a
-  *yearNumber unless \a yearNumber is null (the default).
+    Returns the week number (1 to 53), and stores the year in \a
+    *yearNumber unless \a yearNumber is null (the default).
 
-  Returns 0 if the date is invalid.
+    Returns 0 if the date is invalid.
 
-  In accordance with ISO 8601, weeks start on Monday and the first
-  Thursday of a year is always in week 1 of that year. Most years
-  have 52 weeks, but some have 53.
+    In accordance with ISO 8601, weeks start on Monday and the first
+    Thursday of a year is always in week 1 of that year. Most years
+    have 52 weeks, but some have 53.
 
-  \a *yearNumber is not always the same as year(). For example, 1
-  January 2000 has week number 52 in the year 1999, and 31 December
-  2002 has week number 1 in the year 2003.
+    \a *yearNumber is not always the same as year(). For example, 1
+    January 2000 has week number 52 in the year 1999, and 31 December
+    2002 has week number 1 in the year 2003.
 
-  \sa isValid()
+    \legalese
+
+    Copyright (c) 1989 The Regents of the University of California.
+    All rights reserved.
+
+    Redistribution and use in source and binary forms are permitted
+    provided that the above copyright notice and this paragraph are
+    duplicated in all such forms and that any documentation,
+    advertising materials, and other materials related to such
+    distribution and use acknowledge that the software was developed
+    by the University of California, Berkeley.  The name of the
+    University may not be used to endorse or promote products derived
+    from this software without specific prior written permission.
+    THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
+    IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
+    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+
+    \sa isValid()
 */
 
 int QDate::weekNumber( int *yearNumber ) const
@@ -423,39 +445,47 @@ int QDate::weekNumber( int *yearNumber ) const
     if ( !isValid() )
 	return 0;
 
-    int dow = dayOfWeek();
-    int doy = dayOfYear();
-    int currYear = year();
-    int jan1WeekDay = QDate( currYear, 1, 1 ).dayOfWeek();
-    int yearNum;
-    int weekNum;
+    int	year = QDate::year();
+    int	yday = dayOfYear() - 1;
+    int	wday = dayOfWeek();
+    if (wday == 7)
+    	wday = 0;
+    int	w;
 
-    if ( doy <= (8 - jan1WeekDay) && jan1WeekDay > 4 ) {
-	yearNum = currYear - 1;
-	weekNum = 52;
-	if ( jan1WeekDay == 5 ||
-	     (jan1WeekDay == 6 && QDate::leapYear(yearNum)) )
-	    weekNum++;
-    } else {
-	int totalDays = 365;
-	if ( QDate::leapYear(currYear) )
-	    totalDays++;
+    for (;;) {
+	int len;
+	int bot;
+	int top;
 
-	if ( (totalDays - doy < 4 - dow)
-	     || (jan1WeekDay == 7 && totalDays - doy < 3) ) {
-	    yearNum = currYear + 1;
-	    weekNum = 1;
-	} else {
-	    int j = doy + ( 7 - dow ) + ( jan1WeekDay - 1 );
-	    yearNum = currYear;
-	    weekNum = j / 7;
-	    if ( jan1WeekDay > 4 )
-		weekNum--;
+	len = leapYear(year) ? 366 : 365;
+	/*
+	** What yday (-3 ... 3) does
+	** the ISO year begin on?
+	*/
+	bot = ((yday + 11 - wday) % 7) - 3;
+	/*
+	** What yday does the NEXT
+	** ISO year begin on?
+	*/
+	top = bot - (len % 7);
+	if (top < -3)
+    	    top += 7;
+	top += len;
+	if (yday >= top) {
+	    ++year;
+    	    w = 1;
+	    break;
 	}
+	if (yday >= bot) {
+	    w = 1 + ((yday - bot) / 7);
+	    break;
+	}
+	--year;
+	yday += leapYear(year) ? 366 : 365;
     }
-    if ( yearNumber )
-	*yearNumber = yearNum;
-    return weekNum;
+    if (yearNumber != 0)
+    	*yearNumber = year;
+    return w;
 }
 
 /*!
@@ -480,7 +510,7 @@ QString QDate::shortMonthName( int month )
 {
 #if defined(QT_CHECK_RANGE)
     if ( month < 1 || month > 12 ) {
-	qWarning( "QDate::shortMonthName: Parameter out ouf range." );
+	qWarning( "QDate::shortMonthName: Parameter out ouf range" );
 	month = 1;
     }
 #endif
@@ -526,7 +556,7 @@ QString QDate::longMonthName( int month )
 {
 #if defined(QT_CHECK_RANGE)
     if ( month < 1 || month > 12 ) {
-	qWarning( "QDate::longMonthName: Parameter out ouf range." );
+	qWarning( "QDate::longMonthName: Parameter out ouf range" );
 	month = 1;
     }
 #endif
@@ -580,7 +610,7 @@ QString QDate::shortDayName( int weekday )
 {
 #if defined(QT_CHECK_RANGE)
     if ( weekday < 1 || weekday > 7 ) {
-	qWarning( "QDate::shortDayName: Parameter out of range." );
+	qWarning( "QDate::shortDayName: Parameter out of range" );
 	weekday = 1;
     }
 #endif
@@ -628,7 +658,7 @@ QString QDate::longDayName( int weekday )
 {
 #if defined(QT_CHECK_RANGE)
     if ( weekday < 1 || weekday > 7 ) {
-	qWarning( "QDate::longDayName: Parameter out of range." );
+	qWarning( "QDate::longDayName: Parameter out of range" );
 	weekday = 1;
     }
 #endif
@@ -685,10 +715,14 @@ QString QDate::longDayName( int weekday )
     If \a f is \c Qt::LocalDate, the string format depends on the
     locale settings of the system.
 
+    If the date is an invalid date, then QString::null will be returned.
+
     \sa shortDayName(), shortMonthName()
 */
 QString QDate::toString( Qt::DateFormat f ) const
 {
+    if ( !isValid() )
+	return QString::null;
     int y, m, d;
     julianToGregorian( jd, y, m, d );
     switch ( f ) {
@@ -784,7 +818,9 @@ QString QDate::toString( Qt::DateFormat f ) const
     \row \i ddd MMMM d yy \i11 Sun July 20 69
     \endtable
 
-    \sa QDate::toString() QTime::toString()
+    If the date is an invalid date, then QString::null will be returned.
+
+    \sa QDateTime::toString() QTime::toString()
 
 */
 QString QDate::toString( const QString& format ) const
@@ -896,7 +932,7 @@ QDate QDate::addYears( int nyears ) const
     y += nyears;
 
     QDate tmp(y,m,1);
-    
+
     if( d > tmp.daysInMonth() )
 	d = tmp.daysInMonth();
 
@@ -1034,9 +1070,11 @@ QDate QDate::fromString( const QString& s, Qt::DateFormat f )
 {
     if ( ( s.isEmpty() ) || ( f == Qt::LocalDate ) ) {
 #if defined(QT_CHECK_RANGE)
-	qWarning( "QDate::fromString: Parameter out of range." );
+	qWarning( "QDate::fromString: Parameter out of range" );
 #endif
-	return QDate();
+	QDate d;
+	d.jd = 0;
+	return d;
     }
     switch ( f ) {
     case Qt::ISODate:
@@ -1081,8 +1119,10 @@ QDate QDate::fromString( const QString& s, Qt::DateFormat f )
 	    }
 #if defined(QT_CHECK_RANGE)
 	    if ( month < 1 || month > 12 ) {
-		qWarning( "QDate::fromString: Parameter out of range." );
-		month = 1;
+		qWarning( "QDate::fromString: Parameter out of range" );
+		QDate d;
+		d.jd = 0;
+		return d;
 	    }
 #endif
 	    int day = s.mid( dayPos, 2 ).stripWhiteSpace().toInt();
@@ -1342,10 +1382,15 @@ int QTime::msec() const
 
     If \a f is Qt::LocalDate, the string format depends on the locale
     settings of the system.
+
+    If the time is an invalid time, then QString::null will be returned.
 */
 
 QString QTime::toString( Qt::DateFormat f ) const
 {
+    if ( !isValid() )
+	return QString::null;
+
     switch ( f ) {
     case Qt::LocalDate:
 	{
@@ -1421,7 +1466,9 @@ QString QTime::toString( Qt::DateFormat f ) const
     \row \i h:m:s ap	    \i11 2:13:9 pm
     \endtable
 
-    \sa QDate::toString() QTime::toString()
+    If the time is an invalid time, then QString::null will be returned.
+
+    \sa QDate::toString() QDateTime::toString()
 */
 QString QTime::toString( const QString& format ) const
 {
@@ -1521,7 +1568,8 @@ QTime QTime::addMSecs( int ms ) const
     is negative if \a t is earlier than this time).
 
     Because QTime measures time within a day and there are 86400
-    seconds in a day, the result is always between -86400 and 86400s.
+    seconds in a day, the result is always between -86400000 and
+    86400000 msec.
 
     \sa secsTo()
 */
@@ -1613,9 +1661,11 @@ QTime QTime::fromString( const QString& s, Qt::DateFormat f )
 {
     if ( ( s.isEmpty() ) || ( f == Qt::LocalDate ) ) {
 #if defined(QT_CHECK_RANGE)
-	qWarning( "QTime::fromString: Parameter out of range." );
+	qWarning( "QTime::fromString: Parameter out of range" );
 #endif
-	return QTime();
+	QTime t;
+	t.ds = MSECS_PER_DAY;
+	return t;
     }
 
     int hour( s.mid( 0, 2 ).toInt() );
@@ -2043,13 +2093,17 @@ void QDateTime::setTime_t( uint secsSince1Jan1970UTC, Qt::TimeSpec ts )
     If \a f is \c Qt::LocalDate, the string format depends on the
     locale settings of the system.
 
-    If the format \a f is invalid, toString() returns a null string.
+    If the format \a f is invalid or the datetime is invalid, toString()
+    returns a null string.
 
     \sa QDate::toString() QTime::toString()
 */
 
 QString QDateTime::toString( Qt::DateFormat f ) const
 {
+    if ( !isValid() )
+	return QString::null;
+
     if ( f == Qt::ISODate ) {
 	return d.toString( Qt::ISODate ) + "T" + t.toString( Qt::ISODate );
     }
@@ -2157,6 +2211,8 @@ QString QDateTime::toString( Qt::DateFormat f ) const
     \row \i h:m:s ap	    \i11 2:13:9 pm
     \endtable
 
+    If the datetime is an invalid datetime, then QString::null will be returned.
+
     \sa QDate::toString() QTime::toString()
 */
 QString QDateTime::toString( const QString& format ) const
@@ -2260,7 +2316,7 @@ int QDateTime::daysTo( const QDateTime &dt ) const
     Example:
     \code
     QDateTime dt = QDateTime::currentDateTime();
-    QDateTime xmas( QDate(dt.year(),12,24), QTime(17,00) );
+    QDateTime xmas( QDate(dt.date().year(),12,24), QTime(17,00) );
     qDebug( "There are %d seconds to Christmas", dt.secsTo(xmas) );
     \endcode
 
@@ -2392,7 +2448,9 @@ QDateTime QDateTime::fromString( const QString& s, Qt::DateFormat f )
 #if defined(QT_CHECK_RANGE)
 	qWarning( "QDateTime::fromString: Parameter out of range" );
 #endif
-	return QDateTime();
+	QDateTime dt;
+	dt.d.jd = 0;
+	return dt;
     }
     if ( f == Qt::ISODate ) {
 	return QDateTime( QDate::fromString( s.mid(0,10), Qt::ISODate ),
@@ -2420,8 +2478,10 @@ QDateTime QDateTime::fromString( const QString& s, Qt::DateFormat f )
 	}
 #if defined(QT_CHECK_RANGE)
 	if ( month < 1 || month > 12 ) {
-	    qWarning( "QDateTime::fromString: Parameter out of range." );
-	    month = 1;
+	    qWarning( "QDateTime::fromString: Parameter out of range" );
+	    QDateTime dt;
+	    dt.d.jd = 0;
+	    return dt;
 	}
 #endif
 	int day = s.mid( 8, 2 ).simplifyWhiteSpace().toInt();
@@ -2429,7 +2489,7 @@ QDateTime QDateTime::fromString( const QString& s, Qt::DateFormat f )
 	QDate date( year, month, day );
 	QTime time;
 	int hour, minute, second;
-	int pivot = s.find( QRegExp("[0-9][0-9]:[0-9][0-9]:[0-9][0-9]") );
+	int pivot = s.find( QRegExp(QString::fromLatin1("[0-9][0-9]:[0-9][0-9]:[0-9][0-9]")) );
 	if ( pivot != -1 ) {
 	    hour = s.mid( pivot, 2 ).toInt();
 	    minute = s.mid( pivot+3, 2 ).toInt();
