@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qwaitcondition_unix.cpp   3.2.3   edited May 21 00:40 $
+** $Id: qt/qwaitcondition_unix.cpp   3.3.3   edited Jan 27 2004 $
 **
 ** QWaitCondition class for Unix
 **
@@ -224,8 +224,7 @@ void QWaitCondition::wakeAll()
 */
 bool QWaitCondition::wait(unsigned long time)
 {
-    pthread_mutex_t mutex;
-    pthread_mutex_init( &mutex, 0 );
+    static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
     pthread_mutex_lock( &mutex );
 
     int ret;
@@ -248,7 +247,6 @@ bool QWaitCondition::wait(unsigned long time)
 #endif
 
     pthread_mutex_unlock( &mutex );
-    pthread_mutex_destroy( &mutex );
 
     return (ret == 0);
 }
