@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qdockarea.cpp   3.2.3   edited Jun 5 11:09 $
+** $Id: qt/qdockarea.cpp   3.3.3   edited Nov 5 2003 $
 **
 ** Implementation of the QDockArea class
 **
@@ -897,7 +897,7 @@ void QDockArea::removeDockWindow( QDockWindow *w, bool makeFloating, bool swap,
 	dockWindows->at( i )->setNewLine( TRUE );
     if ( makeFloating ) {
 	QWidget *p = parentWidget() ? parentWidget() : topLevelWidget();
-	dockWindow->reparent( p, WType_Dialog | WStyle_Customize | WStyle_NoBorder, QPoint( 0, 0 ), FALSE );
+	dockWindow->reparent( p, WType_Dialog | WStyle_Customize | WStyle_NoBorder | WStyle_Tool, QPoint( 0, 0 ), FALSE );
     }
     if ( swap )
 	dockWindow->resize( dockWindow->height(), dockWindow->width() );
