@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qworkspace.cpp   3.2.3   edited Oct 29 16:32 $
+** $Id: qt/qworkspace.cpp   3.3.3   edited Jul 16 16:43 $
 **
 ** Implementation of the QWorkspace class
 **
@@ -58,6 +58,7 @@
 #include "qimage.h"
 #include "qscrollbar.h"
 #include "qstyle.h"
+#include "qbitmap.h"
 
 // magic non-mdi things
 #include "qtimer.h"
@@ -148,7 +149,6 @@ public:
     bool isActive() const;
 
     void adjustToFullscreen();
-    void adjustSize();
 
     void setStatusBar(QStatusBar *);
     QWidget* windowWidget() const;
@@ -431,8 +431,8 @@ void QWorkspace::childEvent( QChildEvent * e)
 	QWorkspaceChild* child = new QWorkspaceChild( w, this, "qt_workspacechild" );
 	child->installEventFilter( this );
 
-	connect( child, SIGNAL( popupOperationMenu( const QPoint& ) ),
-		 this, SLOT( popupOperationMenu( const QPoint& ) ) );
+	connect( child, SIGNAL( popupOperationMenu(const QPoint&) ),
+		 this, SLOT( popupOperationMenu(const QPoint&) ) );
 	connect( child, SIGNAL( showOperationMenu() ),
 		 this, SLOT( showOperationMenu() ) );
 	d->windows.append( child );
@@ -526,6 +526,9 @@ void QWorkspace::activateWindow( QWidget* w, bool change_focus )
     // Then activate the new one, so the focus is stored correctly
     d->active->setActive( TRUE );
 
+    if (!d->active)
+	return;
+
     if ( d->maxWindow && d->maxWindow != d->active && d->active->windowWidget() &&
 	 d->active->windowWidget()->testWFlags( WStyle_MinMax ) &&
 	 !d->active->windowWidget()->testWFlags( WStyle_Tool ) ) {
@@ -534,17 +537,19 @@ void QWorkspace::activateWindow( QWidget* w, bool change_focus )
 #ifndef QT_NO_WIDGET_TOPEXTRA
 	    if ( w->icon() ) {
 		QPixmap pm(*w->icon());
-		if(pm.width() != 14 || pm.height() != 14) {
+		int iconSize = d->maxtools->size().height();
+		if(pm.width() > iconSize || pm.height() > iconSize) {
 		    QImage im;
 		    im = pm;
-		    pm = im.smoothScale( 14, 14 );
+		    pm = im.smoothScale( QMIN(iconSize, pm.width()), QMIN(iconSize, pm.height()) );
 		}
 		d->maxtools->setPixmap( pm );
 	    } else
 #endif
 	    {
 		QPixmap pm(14,14);
-		pm.fill( white );
+		pm.fill( color1 );
+		pm.setMask(pm.createHeuristicMask());
 		d->maxtools->setPixmap( pm );
 	    }
 	}
@@ -616,7 +621,7 @@ void QWorkspace::place( QWidget* w)
 	    while ( it.current () ) {
 		l = it.current();
 		++it;
-		
+
 		if ( d->maxWindow == l )
 		    r2 = d->maxRestore;
 		else
@@ -797,7 +802,7 @@ void QWorkspace::handleUndock(QDockWindow *w)
     QPoint wpos(maxRect.left(), maxRect.top());
     int possible = 0;
     if(!::qt_cast<QToolBar*>(w)) {
-	struct place_score { int o, x, y; } score;
+	struct place_score { int o, x, y; } score = {0, 0, 0};
 	int left = 1, x = wpos.x(), y = wpos.y();
 	QPtrListIterator<QDockWindow> it( d->dockwindows );
 	while(1) {
@@ -1053,9 +1058,17 @@ void QWorkspace::showEvent( QShowEvent *e )
     if ( d->becomeActive ) {
 	activateWindow( d->becomeActive );
 	d->becomeActive = 0;
-    }
-    else if ( d->windows.count() > 0 && !d->active )
+    } else if ( d->windows.count() > 0 && !d->active ) {
 	activateWindow( d->windows.first()->windowWidget() );
+    }
+
+    // force a frame repaint - this is a workaround for what seems to be a bug
+    // introduced when changing the QWidget::show() implementation. Might be
+    // a windows bug as well though.
+    for (QPtrListIterator<QWorkspaceChild> it( d->windows ); it.current(); ++it ) {
+	QWorkspaceChild* c = it.current();
+        QApplication::postEvent(c, new QPaintEvent(c->rect(), TRUE));
+    }
 
     updateWorkspace();
 }
@@ -1097,12 +1110,15 @@ void QWorkspace::minimizeWindow( QWidget* w)
 		c->widgetResizeHandler->setActive( TRUE );
 	    }
 	}
+
 	insertIcon( c->iconWidget() );
 	c->hide();
 	if ( wasMax )
 	    c->setGeometry( d->maxRestore );
 	d->focus.append( c );
 
+        activateWindow(w);
+
 	setUpdatesEnabled( TRUE );
 	updateWorkspace();
 
@@ -1217,13 +1233,15 @@ void QWorkspace::maximizeWindow( QWidget* w)
 void QWorkspace::showWindow( QWidget* w)
 {
     if ( d->maxWindow && w->testWFlags( WStyle_Maximize ) && !w->testWFlags( WStyle_Tool) )
-	maximizeWindow( w );
+	maximizeWindow(w);
+    else if (w->isMinimized() && !w->testWFlags(WStyle_Tool))
+	minimizeWindow(w);
     else if ( !w->testWFlags( WStyle_Tool ) )
-	normalizeWindow( w );
+	normalizeWindow(w);
     else
 	w->parentWidget()->show();
     if ( d->maxWindow )
-	d->maxWindow->raise();
+	d->maxWindow->internalRaise();
     updateWorkspace();
 }
 
@@ -1389,6 +1407,7 @@ bool QWorkspace::eventFilter( QObject *o, QEvent * e)
 #endif
 	    }
 	}
+	updateWorkspace();
 	break;
     case QEvent::Show:
 	if ( o->isA("QWorkspaceChild") && !d->focus.containsRef( (QWorkspaceChild*)o ) )
@@ -1403,7 +1422,7 @@ bool QWorkspace::eventFilter( QObject *o, QEvent * e)
 	inCaptionChange = TRUE;
 	if ( o == topLevelWidget() ) {
 	    QWidget *tlw = (QWidget*)o;
-	    if ( !d->maxWindow 
+	    if ( !d->maxWindow
 		|| tlw->caption() != tr("%1 - [%2]").arg(d->topCaption).arg(d->maxWindow->caption()) )
 		d->topCaption = tlw->caption();
 	}
@@ -1472,7 +1491,7 @@ void QWorkspace::showMaximizeControls()
 	d->maxcontrols = new QFrame( topLevelWidget(), "qt_maxcontrols" );
 	QHBoxLayout* l = new QHBoxLayout( d->maxcontrols,
 					  d->maxcontrols->frameWidth(), 0 );
-	if ( d->maxWindow->windowWidget() && 
+	if ( d->maxWindow->windowWidget() &&
 	     d->maxWindow->windowWidget()->testWFlags(WStyle_Minimize) ) {
 	    QToolButton* iconB = new QToolButton( d->maxcontrols, "iconify" );
 #ifndef QT_NO_TOOLTIP
@@ -1530,17 +1549,19 @@ void QWorkspace::showMaximizeControls()
 #ifndef QT_NO_WIDGET_TOPEXTRA
 	if ( d->active->windowWidget() && d->active->windowWidget()->icon() ) {
 	    QPixmap pm(*d->active->windowWidget()->icon());
-	    if(pm.width() != 14 || pm.height() != 14) {
+	    int iconSize = d->maxcontrols->size().height();
+	    if(pm.width() > iconSize || pm.height() > iconSize) {
 		QImage im;
 		im = pm;
-		pm = im.smoothScale( 14, 14 );
+		pm = im.smoothScale( QMIN(iconSize, pm.width()), QMIN(iconSize, pm.height()) );
 	    }
 	    d->maxtools->setPixmap( pm );
 	} else
 #endif
 	{
 	    QPixmap pm(14,14);
-	    pm.fill( white );
+	    pm.fill( color1 );
+	    pm.setMask(pm.createHeuristicMask());
 	    d->maxtools->setPixmap( pm );
 	}
 	d->menuId = b->insertItem( d->maxtools, -1, 0 );
@@ -1685,7 +1706,10 @@ void QWorkspace::toolMenuAboutToShow()
     if ( !d->active || !d->active->windowWidget() )
 	return;
 
-    d->toolPopup->setItemEnabled( 3, !d->active->shademode );
+    QWidget *windowWidget = d->active->windowWidget();
+    bool canResize = windowWidget->maximumSize() != windowWidget->minimumSize();
+
+    d->toolPopup->setItemEnabled( 3, !d->active->shademode && canResize );
     if ( d->active->shademode )
 	d->toolPopup->changeItem( 6,
 				  QIconSet(style().stylePixmap(QStyle::SP_TitleBarUnshadeButton)), tr("&Unshade") );
@@ -1760,7 +1784,7 @@ void QWorkspace::activateNextWindow()
     if ( d->focus.at( a ) )
 	activateWindow( d->focus.at( a )->windowWidget(), FALSE );
     else
-	d->active = 0;
+	activateWindow(0);
 }
 
 void QWorkspace::activatePreviousWindow()
@@ -1836,11 +1860,8 @@ void QWorkspace::cascade()
     // make a list of all relevant mdi clients
     QPtrList<QWorkspaceChild> widgets;
     QWorkspaceChild* wc = 0;
-    for ( wc = d->windows.first(); wc; wc = d->windows.next() )
-	if ( wc->iconw )
-	    normalizeWindow( wc->windowWidget() );
     for ( wc = d->focus.first(); wc; wc = d->focus.next() )
-	if ( wc->windowWidget()->isVisibleTo( this ) && !wc->windowWidget()->testWFlags( WStyle_Tool ) )
+	if ( wc->windowWidget()->isVisibleTo( this ) && !wc->windowWidget()->testWFlags( WStyle_Tool ) && !wc->iconw )
 	    widgets.append( wc );
 
     int x = 0;
@@ -1853,15 +1874,12 @@ void QWorkspace::cascade()
 	QWorkspaceChild *child = it.current();
 	++it;
 	child->setUpdatesEnabled( FALSE );
-	bool hasSizeHint = FALSE;
 	QSize prefSize = child->windowWidget()->sizeHint().expandedTo( child->windowWidget()->minimumSizeHint() );
 
 	if ( !prefSize.isValid() )
-	    prefSize = QSize( width() - children * xoffset, height() - children * yoffset );
-	else
-	    hasSizeHint = TRUE;
+	    prefSize = child->windowWidget()->size();
 	prefSize = prefSize.expandedTo( child->windowWidget()->minimumSize() ).boundedTo( child->windowWidget()->maximumSize() );
-	if ( hasSizeHint )
+	if (prefSize.isValid())
 	    prefSize += QSize( child->baseSize().width(), child->baseSize().height() );
 
 	int w = prefSize.width();
@@ -1917,7 +1935,8 @@ void QWorkspace::tile()
 	++it;
 	if ( !c->windowWidget()->isHidden() &&
 	     !c->windowWidget()->testWFlags( WStyle_StaysOnTop ) &&
-	     !c->windowWidget()->testWFlags( WStyle_Tool ) )
+	     !c->windowWidget()->testWFlags( WStyle_Tool ) &&
+             !c->iconw )
 	    n++;
     }
 
@@ -1941,8 +1960,12 @@ void QWorkspace::tile()
     while ( it.current () ) {
 	c = it.current();
 	++it;
-	if ( c->windowWidget()->isHidden() || c->windowWidget()->testWFlags( WStyle_Tool ) )
+	if ( c->iconw || c->windowWidget()->isHidden() || c->windowWidget()->testWFlags( WStyle_Tool ) )
 	    continue;
+        if (!row && !col) {
+            w -= c->baseSize().width();
+            h -= c->baseSize().height();
+        }
 	if ( c->windowWidget()->testWFlags( WStyle_StaysOnTop ) ) {
 	    QPoint p = c->pos();
 	    if ( p.x()+c->width() < 0 )
@@ -1954,21 +1977,22 @@ void QWorkspace::tile()
 	    if ( p.y() > height() )
 		p.setY( height() - c->height() );
 
-
 	    if ( p != c->pos() )
 		c->QFrame::move( p );
 	} else {
 	    c->showNormal();
 	    qApp->sendPostedEvents( 0, QEvent::ShowNormal );
 	    used[row*cols+col] = TRUE;
+            QSize sz(w, h + add ? h : 0);
+            QSize bsize(c->baseSize());
+            sz = sz.expandedTo(c->windowWidget()->minimumSize()).boundedTo(c->windowWidget()->maximumSize());
+            sz += bsize;
 	    if ( add ) {
-		c->setGeometry( col*w, row*h, QMIN( w, c->windowWidget()->maximumWidth()+c->baseSize().width() ),
-					      QMIN( 2*h, c->windowWidget()->maximumHeight()+c->baseSize().height() ) );
+		c->setGeometry(col*w + col*bsize.width(), row*h + row*bsize.height(), sz.width(), sz.height());
 		used[(row+1)*cols+col] = TRUE;
 		add--;
 	    } else {
-		c->setGeometry( col*w, row*h, QMIN( w, c->windowWidget()->maximumWidth()+c->baseSize().width() ),
-					      QMIN( h, c->windowWidget()->maximumHeight()+c->baseSize().height() ) );
+		c->setGeometry(col*w + col*bsize.width(), row*h + row*bsize.height(), sz.width(), sz.height());
 	    }
 	    while( row < rows && col < cols && used[row*cols+col] ) {
 		col++;
@@ -2027,8 +2051,8 @@ QWorkspaceChild::QWorkspaceChild( QWidget* window, QWorkspace *parent,
 		 this, SLOT( showNormal() ) );
 	connect( titlebar, SIGNAL( doMaximize() ),
 		 this, SLOT( showMaximized() ) );
-	connect( titlebar, SIGNAL( popupOperationMenu( const QPoint& ) ),
-		 this, SIGNAL( popupOperationMenu( const QPoint& ) ) );
+	connect( titlebar, SIGNAL( popupOperationMenu(const QPoint&) ),
+		 this, SIGNAL( popupOperationMenu(const QPoint&) ) );
 	connect( titlebar, SIGNAL( showOperationMenu() ),
 		 this, SIGNAL( showOperationMenu() ) );
 	connect( titlebar, SIGNAL( doShade() ),
@@ -2063,13 +2087,13 @@ QWorkspaceChild::QWorkspaceChild( QWidget* window, QWorkspace *parent,
     int th = titlebar ? titlebar->sizeHint().height() : 0;
     if ( titlebar ) {
 #ifndef QT_NO_WIDGET_TOPEXTRA
-	int iconSize = th - frameWidth() * 2;
+	int iconSize = th;
 	if( childWidget->icon() ) {
 	    QPixmap pm(*childWidget->icon());
-	    if(pm.width() != iconSize || pm.height() != iconSize) {
+	    if(pm.width() > iconSize || pm.height() > iconSize) {
 		QImage im;
 		im = pm;
-		pm = im.smoothScale( iconSize, iconSize );
+		pm = im.smoothScale( QMIN(iconSize, pm.width()), QMIN(iconSize, pm.height()) );
 	    }
 	    titlebar->setIcon( pm );
 	}
@@ -2098,7 +2122,7 @@ QWorkspaceChild::QWorkspaceChild( QWidget* window, QWorkspace *parent,
     widgetResizeHandler->setSizeProtection( !parent->scrollBarsEnabled() );
     connect( widgetResizeHandler, SIGNAL( activate() ),
 	     this, SLOT( activate() ) );
-    widgetResizeHandler->setExtraHeight( th + contentsRect().y() );
+    widgetResizeHandler->setExtraHeight( th + contentsRect().y() - 2*frameWidth() );
     if(parent->windowMode() == QWorkspace::TopLevel && isTopLevel()) {
 	move(0, 0);
 	widgetResizeHandler->setActive( FALSE );
@@ -2114,6 +2138,7 @@ QWorkspaceChild::~QWorkspaceChild()
 	delete iconw->parentWidget();
     QWorkspace *workspace = ::qt_cast<QWorkspace*>(parentWidget());
     if ( workspace ) {
+        workspace->d->focus.removeRef(this);
 	if ( workspace->d->active == this )
 	    workspace->activatePrevWindow();
 	if ( workspace->d->maxWindow == this ) {
@@ -2202,7 +2227,7 @@ void QWorkspaceChild::resizeEvent( QResizeEvent * )
 	titlebar->setGeometry( tbrect );
 
 	if ( style().styleHint( QStyle::SH_TitleBar_NoBorder, titlebar ) )
-	    th -= r.y();
+	    th -= frameWidth();
 	cr = QRect( r.x(), r.y() + th + (shademode ? (frameWidth() * 3) : 0),
 		    r.width(), r.height() - th );
     } else {
@@ -2226,10 +2251,8 @@ void QWorkspaceChild::resizeEvent( QResizeEvent * )
 QSize QWorkspaceChild::baseSize() const
 {
     int th = titlebar ? titlebar->sizeHint().height() : 0;
-    if ( !style().styleHint( QStyle::SH_TitleBar_NoBorder, titlebar ) )
-	th += frameWidth();
-    else
-	th -= contentsRect().y();
+    if ( style().styleHint( QStyle::SH_TitleBar_NoBorder, titlebar ) )
+	th -= frameWidth();
     return QSize( 2*frameWidth(), 2*frameWidth() + th );
 }
 
@@ -2237,7 +2260,12 @@ QSize QWorkspaceChild::sizeHint() const
 {
     if ( !childWidget )
 	return QFrame::sizeHint() + baseSize();
-    return childWidget->sizeHint().expandedTo( childWidget->minimumSizeHint() ) + baseSize();
+
+    QSize prefSize = windowWidget()->sizeHint().expandedTo( windowWidget()->minimumSizeHint() );
+    prefSize = prefSize.expandedTo( windowWidget()->minimumSize() ).boundedTo( windowWidget()->maximumSize() );
+    prefSize += baseSize();
+
+    return prefSize;
 }
 
 QSize QWorkspaceChild::minimumSizeHint() const
@@ -2296,6 +2324,9 @@ bool QWorkspaceChild::eventFilter( QObject * o, QEvent * e)
 	     ( windowWidget()->maximumWidth() < parentWidget()->width() ||
 	       windowWidget()->maximumHeight() < parentWidget()->height() ) ) {
 	    windowWidget()->resize( windowWidget()->maximumSize() );
+            ((QWorkspace*)windowWidget())->clearWState(WState_Maximized);
+            if (titlebar)
+                titlebar->repaint(FALSE);
 	    break;
 	}
 	if ( windowWidget()->testWFlags( WStyle_Maximize ) && !windowWidget()->testWFlags( WStyle_Tool ) )
@@ -2338,23 +2369,24 @@ bool QWorkspaceChild::eventFilter( QObject * o, QEvent * e)
 		break;
 
 	    QPixmap pm;
-	    int iconSize = titlebar->size().height() - frameWidth() * 2;
+	    int iconSize = titlebar->size().height();
 #ifndef QT_NO_WIDGET_TOPEXTRA
 	    if ( childWidget->icon() ) {
 		pm = *childWidget->icon();
-		if(pm.width() != iconSize || pm.height() != iconSize) {
+		if(pm.width() > iconSize || pm.height() > iconSize) {
 		    QImage im;
 		    im = pm;
-		    pm = im.smoothScale( iconSize, iconSize );
+		    pm = im.smoothScale( QMIN(iconSize, pm.width()), QMIN(iconSize, pm.height()) );
 		}
-	    } else 
+	    } else
 #endif
 	    {
 		pm.resize( iconSize, iconSize );
-		pm.fill( white );
+		pm.fill( color1 );
+		pm.setMask(pm.createHeuristicMask());
 	    }
 	    titlebar->setIcon( pm );
-	    if ( iconw )
+	    if ( iconw ) 		
 		iconw->setIcon( pm );
 
 	    if ( ws->d->maxWindow != this )
@@ -2590,8 +2622,8 @@ QWidget* QWorkspaceChild::iconWidget() const
 		 this, SLOT( showNormal() ) );
 	connect( iconw, SIGNAL( doMaximize() ),
 		 this, SLOT( showMaximized() ) );
-	connect( iconw, SIGNAL( popupOperationMenu( const QPoint& ) ),
-		 this, SIGNAL( popupOperationMenu( const QPoint& ) ) );
+	connect( iconw, SIGNAL( popupOperationMenu(const QPoint&) ),
+		 this, SIGNAL( popupOperationMenu(const QPoint&) ) );
 	connect( iconw, SIGNAL( showOperationMenu() ),
 		 this, SIGNAL( showOperationMenu() ) );
 	connect( iconw, SIGNAL( doubleClicked() ),
@@ -2601,13 +2633,13 @@ QWidget* QWorkspaceChild::iconWidget() const
     if ( windowWidget() ) {
 	iconw->setCaption( windowWidget()->caption() );
 	if ( windowWidget()->icon() ) {
-	    int iconSize = iconw->sizeHint().height() - frameWidth()*2;
+	    int iconSize = iconw->sizeHint().height();
 
 	    QPixmap pm(*childWidget->icon());
-	    if(pm.width() != iconSize || pm.height() != iconSize) {
+	    if(pm.width() > iconSize || pm.height() > iconSize) {
 		QImage im;
 		im = pm;
-		pm = im.smoothScale( iconSize, iconSize );
+		pm = im.smoothScale( QMIN(iconSize, pm.width()), QMIN(iconSize, pm.height()) );
 	    }
 	    iconw->setIcon( pm );
 	}
@@ -2618,17 +2650,17 @@ QWidget* QWorkspaceChild::iconWidget() const
 
 void QWorkspaceChild::showMinimized()
 {
-    QApplication::postEvent( windowWidget(), new QEvent( QEvent::ShowMinimized ) );
+    windowWidget()->setWindowState(WindowMinimized | windowWidget()->windowState());
 }
 
 void QWorkspaceChild::showMaximized()
 {
-    QApplication::postEvent( windowWidget(), new QEvent( QEvent::ShowMaximized ) );
+    windowWidget()->setWindowState(WindowMaximized | (windowWidget()->windowState() & ~WindowMinimized));
 }
 
 void QWorkspaceChild::showNormal()
 {
-    QApplication::postEvent( windowWidget(), new QEvent( QEvent::ShowNormal ) );
+    windowWidget()->setWindowState(windowWidget()->windowState() & ~(WindowMinimized|WindowMaximized));
 }
 
 void QWorkspaceChild::showShaded()
@@ -2645,6 +2677,7 @@ void QWorkspaceChild::showShaded()
 	shademode = FALSE;
 	resize( shadeRestore );
 	setMinimumSize( shadeRestoreMin );
+	style().polish(this);
     } else {
 	shadeRestore = size();
 	shadeRestoreMin = minimumSize();
@@ -2654,7 +2687,11 @@ void QWorkspaceChild::showShaded()
 	fake->setWState( WState_Minimized );
 	setWState( WState_Minimized );
 
-	resize( width(), titlebar->height() + 2*lineWidth() + 1 );
+	if ( style().styleHint( QStyle::SH_TitleBar_NoBorder ) )
+	    resize( width(), titlebar->height() );
+	else
+	    resize( width(), titlebar->height() + 2*lineWidth() + 1 );
+	style().polish(this);
     }
     titlebar->update();
 }
@@ -2694,18 +2731,6 @@ void QWorkspaceChild::adjustToFullscreen()
     ((QWorkspaceChild*)childWidget)->setWState( WState_Maximized );
 }
 
-void QWorkspaceChild::adjustSize()
-{
-    if ( !testWState(WState_Polished) )
-	polish();
-
-    QSize prefSize = windowWidget()->sizeHint().expandedTo( windowWidget()->minimumSizeHint() );
-    prefSize = prefSize.expandedTo( windowWidget()->minimumSize() ).boundedTo( windowWidget()->maximumSize() );
-    prefSize += baseSize();
-
-    resize( prefSize );
-}
-
 void QWorkspaceChild::setCaption( const QString& cap )
 {
     if ( titlebar )
@@ -2824,7 +2849,7 @@ QRect QWorkspace::updateWorkspace()
 	d->vbar->raise();
 	d->hbar->raise();
 	if ( d->maxWindow )
-	    d->maxWindow->raise();
+	    d->maxWindow->internalRaise();
 
 	QRect r( 0, 0, 0, 0 );
 	QPtrListIterator<QWorkspaceChild> it( d->windows );
