@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qsvgdevice.cpp   3.2.3   edited Aug 12 16:29 $
+** $Id: qt/qsvgdevice.cpp   3.3.3   edited Apr 28 15:55 $
 **
 ** Implementation of the QSvgDevice class
 **
@@ -84,6 +84,10 @@ public:
     PixmapList pixmaps;
     StateList stack;
     int currentClip;
+
+    uint justRestored : 1;
+
+    QMap<QString, QRegion> clipPathTable;
 };
 
 enum ElementType {
@@ -140,6 +144,7 @@ QSvgDevice::QSvgDevice()
 {
     d = new QSvgDevicePrivate;
     d->currentClip = 0;
+    d->justRestored = false;
 }
 
 /*!
@@ -576,6 +581,7 @@ bool QSvgDevice::cmd ( int c, QPainter *painter, QPDevCmdParam *p )
 	    e.setAttribute( "points", str.stripWhiteSpace() );
 	}
 	break;
+#ifndef QT_NO_BEZIER
     case PdcDrawCubicBezier:
 	a = *p[0].ptarr;
 	e = doc.createElement( "path" );
@@ -584,6 +590,7 @@ bool QSvgDevice::cmd ( int c, QPainter *painter, QPDevCmdParam *p )
 		     a[3].x(), a[3].y() );
 	e.setAttribute( "d", str );
 	break;
+#endif
     case PdcDrawText2:
 	e = doc.createElement( "text" );
 	if ( p[0].point->x() )
@@ -648,6 +655,7 @@ bool QSvgDevice::cmd ( int c, QPainter *painter, QPDevCmdParam *p )
     case PdcRestore:
 	current = current.parentNode();
 	dirtyTransform = !pt->worldMatrix().isIdentity();
+        d->justRestored = true;
 	// ### reset dirty flags
 	break;
     case PdcSetBkColor:
@@ -679,30 +687,28 @@ bool QSvgDevice::cmd ( int c, QPainter *painter, QPDevCmdParam *p )
 	break;
     case PdcSetClipRegion:
 	{
+            // We skip the clip after restore, since restoring the clip is done automatically by
+            // the viewer as part of the tree structure. It doesn't hurt to write the region
+            // out, but it doubles the number of clipregions defined in the final svg.
+            if (d->justRestored) {
+                d->justRestored = false;
+                return TRUE;
+            }
+
+            QMemArray<QRect> rects = p[0].rgn->rects();
+            if (rects.count() == 0)
+                return TRUE;
 	    d->currentClip++;
 	    e = doc.createElement( "clipPath" );
 	    e.setAttribute( "id", QString("clip%1").arg(d->currentClip) );
-	    QRect br = p[0].rgn->boundingRect();
-	    QDomElement ce;
-	    if ( p[0].rgn->rects().count() == 1 ) {
-		// Then it's just a rect, boundingRect() will do
-		ce = doc.createElement( "rect" );
-		ce.setAttribute( "x", br.x() );
-		ce.setAttribute( "y", br.y() );
-		ce.setAttribute( "width", br.width() );
-		ce.setAttribute( "height", br.height() );
-	    } else {
-		// It's an ellipse, calculate the ellipse
-		// from the boundingRect()
-		ce = doc.createElement( "ellipse" );
-		double cx = br.x() + (br.width() / 2.0);
-		double cy = br.y() + (br.height() / 2.0);
-		ce.setAttribute( "cx", cx );
-		ce.setAttribute( "cy", cy );
-		ce.setAttribute( "rx", cx - br.x() );
-		ce.setAttribute( "ry", cy - br.y() );
+            for (int i=0; i<(int)rects.count(); ++i) {
+                QDomElement ce = doc.createElement("rect");
+                ce.setAttribute( "x", rects.at(i).x() );
+                ce.setAttribute( "y", rects.at(i).y() );
+                ce.setAttribute( "width", rects.at(i).width() );
+                ce.setAttribute( "height", rects.at(i).height() );
+                e.appendChild(ce);
 	    }
-	    e.appendChild( ce );
 	    break;
 	}
     default:
@@ -986,30 +992,38 @@ bool QSvgDevice::play( const QDomNode &node )
 	    break;
 	case ClipElement:
 	    {
-		restoreAttributes(); // To ensure the clip rect is saved, we need to restore now
 		QDomNode child = node.firstChild();
-		QDomNamedNodeMap childAttr = child.attributes();
-		if ( child.nodeName() == "rect" ) {
-		    QRect r;
-		    r.setX(lenToInt( childAttr, "x" ));
-		    r.setY(lenToInt( childAttr, "y" ));
-		    r.setWidth(lenToInt( childAttr, "width" ));
-		    r.setHeight(lenToInt( childAttr, "height" ));
-		    pt->setClipRect( r, QPainter::CoordPainter );
-		} else if ( child.nodeName() == "ellipse" ) {
-		    QRect r;
-		    int x = lenToInt( childAttr, "cx" );
-		    int y = lenToInt( childAttr, "cy" );
-		    int width = lenToInt( childAttr, "rx" );
-		    int height = lenToInt( childAttr, "ry" );
-		    r.setX( x - width );
-		    r.setY( y - height );
-		    r.setWidth( width * 2 );
-		    r.setHeight( height * 2 );
-		    QRegion rgn( r, QRegion::Ellipse );
-		    pt->setClipRegion( rgn, QPainter::CoordPainter );
-		}
-		break;
+                QRegion region;
+                while (!child.isNull()) {
+                    QDomNamedNodeMap childAttr = child.attributes();
+                    if ( child.nodeName() == "rect" ) {
+                        QRect r;
+                        r.setX(lenToInt( childAttr, "x" ));
+                        r.setY(lenToInt( childAttr, "y" ));
+                        r.setWidth(lenToInt( childAttr, "width" ));
+                        r.setHeight(lenToInt( childAttr, "height" ));
+                        region |= r;
+                    } else if ( child.nodeName() == "ellipse" ) {
+                        QRect r;
+                        int x = lenToInt( childAttr, "cx" );
+                        int y = lenToInt( childAttr, "cy" );
+                        int width = lenToInt( childAttr, "rx" );
+                        int height = lenToInt( childAttr, "ry" );
+                        r.setX( x - width );
+                        r.setY( y - height );
+                        r.setWidth( width * 2 );
+                        r.setHeight( height * 2 );
+                        QRegion rgn( r, QRegion::Ellipse );
+                        region |= rgn;
+                    }
+                    child = child.nextSibling();
+                }
+                // Store the region in a named map so that it can be used when the
+                // group node is entered.
+                QString idString = attr.namedItem("id").nodeValue();
+                if (!idString.isEmpty())
+                    d->clipPathTable[idString] = region;
+                break;
 	    }
 	case InvalidElement:
 	    qWarning( "QSvgDevice::play: unknown element type " +
@@ -1017,7 +1031,6 @@ bool QSvgDevice::play( const QDomNode &node )
 	    break;
 	};
 
-    if ( t != ClipElement )
 	restoreAttributes();
 
     return TRUE;
@@ -1248,6 +1261,15 @@ void QSvgDevice::setStyleProperty( const QString &prop, const QString &val,
 	    *talign = Qt::AlignRight;
 	else
 	    *talign = Qt::AlignLeft;
+    } else if ( prop == "clip-path" ) {
+        if (val.startsWith("url(#")) {
+            QString clipName = val.mid(5, val.length() - 6);
+            if (!clipName.isEmpty()) {
+                QRegion clipRegion = d->clipPathTable[clipName];
+                if (!clipRegion.isEmpty())
+                    pt->setClipRegion(pt->clipRegion() & clipRegion, QPainter::CoordPainter);
+            }
+        }
     }
 }
 
@@ -1399,6 +1421,7 @@ void QSvgDevice::drawPath( const QString &data )
 	    y = arg[ 0 ] + offsetY;
 	    path.setPoint( pcount++, int(x), int(y) );
 	    break;
+#ifndef QT_NO_BEZIER
 	case 5:					// 'C' cubic bezier curveto
 	case 6:					// 'S' smooth shorthand
 	case 7:					// 'Q' quadratic bezier curves
@@ -1441,6 +1464,7 @@ void QSvgDevice::drawPath( const QString &data )
 		path.setPoint( pcount++, bezier[ k ] );
 	    break;
 	}
+#endif // QT_NO_BEZIER
 	case 9:					// 'A' elliptical arc curve
 	    // ### just a straight line
 	    x = arg[ 5 ] + offsetX;
